SystemOrganization addCategory: #'Collections-Tests-BitmapCharacterSet'!

TestCase subclass: #BitmapCharacterSetTest
	instanceVariableNames: 'characters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-Tests-BitmapCharacterSet'!

!BitmapCharacterSetTest methodsFor: 'enumerating' stamp: 'monty 9/1/2015 05:55'!
absentCharactersDo: aBlock
	| previousValue |

	previousValue := 0.
	self charactersDo: [:each |
		previousValue + 1 to: each asciiValue - 1 do: [:i |
			aBlock value: i asCharacter].
		previousValue := each asciiValue].

	previousValue + 1 to: self lastCodePoint - 1 do: [:i |
		aBlock value: i asCharacter].! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 9/1/2015 05:56'!
characters
	^ characters
		ifNil: [| writeStream previousValue offset |
			writeStream := (Array new: 250000) writeStream.
			previousValue := 0.
			writeStream nextPut: previousValue asCharacter.
			offset := 1.
			1 to: self lastCodePoint do: [:i |
				previousValue + offset = i
					ifTrue: [
						writeStream nextPut: i asCharacter.
						previousValue := i.
						offset :=
							offset = 8
								ifTrue: [1]
								ifFalse: [offset + 1]]].
			characters := writeStream contents]! !

!BitmapCharacterSetTest methodsFor: 'enumerating' stamp: 'monty 8/30/2015 06:56'!
charactersDo: aBlock
	self characters do: aBlock! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 3/16/2016 01:37'!
emptySet
	^ self setClass new! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 3/10/2016 10:22'!
lastCodePoint
	^ 16r10FFFF! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 3/16/2016 01:37'!
nonEmptySet
	^ self setClass newFrom: self characters! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 3/16/2016 01:30'!
rangeCharacters
	^ 'abcdefghijklmnopqrstuvwxyz'! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 3/16/2016 01:30'!
rangeStart
	^ self rangeCharacters first! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 3/16/2016 01:30'!
rangeStop
	^ self rangeCharacters last! !

!BitmapCharacterSetTest methodsFor: 'accessing' stamp: 'monty 3/16/2016 01:33'!
setClass
	^ BitmapCharacterSet! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testAdd
	| set |

	set := self emptySet.
	self charactersDo: [:each |
		self
			deny: (set includes: each);
			assert: (set add: each) = each;
			assert: (set includes: each);
			assert: (set add: each) = each;
			assert: (set includes: each)].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:57'!
testAddRangeFromTo
	| set |

	set := self emptySet.
	self "empty range, because from > to"
		assert: (set addRangeFrom: self rangeStop to: self rangeStart) == set;
		assert: set isEmpty.

	self
		assert: (set addRangeFrom: self rangeStart to: self rangeStop) == set;
		assert: set = (self setClass newFrom: self rangeCharacters).

	set := self emptySet.
	self
		assert: (set addRangeFrom: self rangeStart to: self rangeStart) == set;
		assert: set = (self setClass with: self rangeStart).! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testByteCharactersDo
	| set enumerated |

	set := self emptySet.
	enumerated := OrderedCollection new.

	set byteCharactersDo: [:each | enumerated addLast: each].
	self assert: enumerated isEmpty.

	set addAll: self characters.
	set byteCharactersDo: [:each | enumerated addLast: each].
	self assert: enumerated notEmpty.
	enumerated withIndexDo: [:each :i |
		self
			assert: each asciiValue < 256;
			assert: each = (self characters at: i)].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:33'!
testCapacity
	"Since the bitmap is allocated in bytes, the last byte can have excess capacity
	even when a specific capacity is specified."

	self
		assert: (self setClass new: 0) capacity = 256;
		assert: (self setClass new: 256) capacity = 256.
	257 to: 264 do: [:i |
		self assert: (self setClass new: i) capacity = 264].
	265 to: 272 do: [:i |
		self assert: (self setClass new: i) capacity = 272].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testCopy
	| set copy |

	set := self nonEmptySet.
	copy := set copy.
	self
		assert: set = copy;
		deny: set == copy;
		assert: set byteCharacters = copy byteCharacters;
		deny: set byteCharacters == copy byteCharacters;
		assert: set wideCharacters = copy wideCharacters;
		deny: set wideCharacters == copy wideCharacters.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testDo
	| set enumerated |

	set := self emptySet.
	enumerated := OrderedCollection new.

	set do: [:each | enumerated addLast: each].
	self assert: enumerated isEmpty.

	set addAll: self characters.
	set do: [:each | enumerated addLast: each].
	self assert: enumerated size = self characters size.
	enumerated
		with: self characters
		do: [:enumeratedChar :expectedChar |
			self assert: enumeratedChar = expectedChar]! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testEquals
	self
		assert: self emptySet = self emptySet;
		assert: self nonEmptySet = self nonEmptySet;
		deny: self emptySet = self nonEmptySet;
		deny: self nonEmptySet = self emptySet! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testHasWideCharacters
	| set |

	set := self emptySet.
	self deny: set hasWideCharacters.

	set add: 255 asCharacter.
	self deny: set hasWideCharacters.

	set add: 256 asCharacter.
	self assert: set hasWideCharacters.

	set remove: 256 asCharacter.
	self deny: set hasWideCharacters.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testHash
	self
		assert: self emptySet hash = self emptySet hash;
		assert: self nonEmptySet hash = self nonEmptySet hash! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testIncludes
	| set |

	set := self emptySet.
	self
		charactersDo: [:each | self deny: (set includes: each)];
		absentCharactersDo: [:each | self deny: (set includes: each)].

	set := self nonEmptySet.
	self
		charactersDo: [:each | self assert: (set includes: each)];
		absentCharactersDo: [:each | self deny: (set includes: each)].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testIncludesRangeFromTo
	| set |

	set := self emptySet.
	self rangeCharacters do: [:each |
		self
			deny: (set includesRangeFrom: self rangeStart to: self rangeStop);
			assert: (set includesRangeFrom: self rangeStop to: self rangeStart).
		set add: each].
	self
		assert: (set includesRangeFrom: self rangeStart to: self rangeStop);
		assert: (set includesRangeFrom: self rangeStop to: self rangeStart).! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:33'!
testMaxCapacity
	| maxCapacity set |

	maxCapacity := self lastCodePoint + 1.

	set := self setClass new: maxCapacity.
	self assert: set capacity = maxCapacity.

	set := self setClass new: maxCapacity + 1.
	self assert: set capacity = maxCapacity.

	set := self setClass new: maxCapacity * 2.
	self assert: set capacity = maxCapacity.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testMaxCharacter
	| set |

	set := self emptySet.
	self
		shouldnt: [set add: self lastCodePoint asCharacter]
		raise: Error.
	self assert: (set includes: self lastCodePoint asCharacter).
	self
		should: [set add: (self lastCodePoint + 1) asCharacter]
		raise: Error.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:33'!
testNewFrom
	| set newSet |

	set := self setClass newFrom: self characters.
	self assert: set class == self setClass.
	self charactersDo: [:each | self assert: (set includes: each)].

	newSet := self setClass newFrom: set.
	self
		assert: newSet = set;
		deny: newSet == set.! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testRemoveAll
	| set capacity byteCharactersSize wideCharactersSize|

	set := self nonEmptySet.
	capacity := set capacity.
	byteCharactersSize := set byteCharacters size.
	wideCharactersSize := set wideCharacters size.
	self
		assert: set removeAll == set;
		assert: set size = 0;
		assert: set capacity = capacity;
		assert: set byteCharacters size = byteCharactersSize;
		assert: (set byteCharacters allSatisfy: [:each | each = false]);
		assert: set wideCharacters size = wideCharactersSize;
		assert: (set wideCharacters allSatisfy: [:each | each = 0]).! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testRemoveIfAbsent
	| set |

	set := self nonEmptySet.
	self absentCharactersDo: [:each |
		self assert: (set remove: each ifAbsent: [nil]) isNil].
	self charactersDo: [:each |
		self assert: (set remove: each ifAbsent: [nil]) = each.
		self assert: (set remove: each ifAbsent: [nil]) isNil].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:57'!
testRemoveRangeFromTo
	| set |

	set := self emptySet.
	self "empty range, because from > to"
		assert: (set removeRangeFrom: self rangeStop to: self rangeStart) == set;
		assert: set isEmpty.

	set addRangeFrom: self rangeStart to: self rangeStop.
	self
		assert: (set removeRangeFrom: self rangeStart to: self rangeStop) == set;
		assert: set isEmpty.

	self rangeCharacters do: [:each |
		set := self setClass newFrom: self rangeCharacters.
		self
			assert: (set removeRangeFrom: each to: each) == set;
			assert:
				set =
					(self setClass newFrom: (self rangeCharacters copyWithout: each)).
		self
			should: [set removeRangeFrom: each to: each]
			raise: Error].! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testSize
	| set size |

	set := self emptySet.
	size := 0.
	self charactersDo: [:each |
		self assert: set size = size.
		set add: each.
		size := size + 1.
		self assert: set size = size.

		"Adding the same character shouldn't alter the size"
		set add: each.
		self assert: set size = size.].

	self charactersDo: [:each |
		set remove: each.
		size := size - 1.
		self assert: set size = size].	! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:33'!
testWideCharacters
	0 to: 7 do: [:i | | set character |
		set := self setClass new.
		character := (256 + i) asCharacter.
		self assert: set wideCharacters isNil.
		set add: character.
		self assert: set wideCharacters first = (16r80 bitShift: i negated).
		set remove: character.
		self assert: set wideCharacters first = 0].
	0 to: 7 do: [:i | | set character |
		set := self setClass new.
		character := (256 + 8 + i) asCharacter.
		self assert: set wideCharacters isNil.
		set add: character.
		self assert: set wideCharacters second = (16r80 bitShift: i negated).
		set remove: character.
		self assert: set wideCharacters second = 0]! !

!BitmapCharacterSetTest methodsFor: 'tests' stamp: 'monty 3/16/2016 01:37'!
testWideCharactersDo
	| set enumerated wideCharOffset |

	set := self emptySet.
	enumerated := OrderedCollection new.

	set wideCharactersDo: [:each | enumerated addLast: each].
	self assert: enumerated isEmpty.

	set addAll: self characters.
	set wideCharactersDo: [:each | enumerated addLast: each].
	self assert: enumerated notEmpty.
	wideCharOffset :=
		(self characters findFirst: [:each | each asciiValue >= 256]) - 1.
	enumerated withIndexDo: [:each :i |
		self
			assert: each asciiValue >= 256;
			assert: each = (self characters at: wideCharOffset + i)].! !
